
/* Autogenerated by mlir-tblgen; don't manually edit. */

#include "mlir-c/Pass.h"

#ifdef __cplusplus
extern "C" {
#endif

// Registration for the entire group
MLIR_CAPI_EXPORTED void mlirRegisterNVGPUPasses(void);


/* Create NVGPU Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateNVGPUOptimizeSharedMemory(void);
MLIR_CAPI_EXPORTED void mlirRegisterNVGPUOptimizeSharedMemory(void);



#ifdef __cplusplus
}
#endif
