/* Autogenerated by mlir-tblgen; don't manually edit. */
//===----------------------------------------------------------------------===//
// NVGPU Group Registration
//===----------------------------------------------------------------------===//

void mlirRegisterNVGPUPasses(void) {
  registerNVGPUPasses();
}

MlirPass mlirCreateNVGPUOptimizeSharedMemory(void) {
  return wrap(mlir::nvgpu::createOptimizeSharedMemoryPass().release());
}
void mlirRegisterNVGPUOptimizeSharedMemory(void) {
  registerOptimizeSharedMemory();
}

